//generate_bg.c
//Takes a raw image and makes a 32x24ch background plane + "extras" (numbers, etc)
//Bryan Topp <betopp@betopp.com> 2015

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAP_WIDTH_CH 32
#define MAP_HEIGHT_CH 25

#define MAXCHAR 400

unsigned char bg_raw[MAP_HEIGHT_CH*8][MAP_WIDTH_CH*8];

unsigned char output_characters[MAXCHAR][8][8];
unsigned short output_charmap[MAP_HEIGHT_CH][MAP_WIDTH_CH];


int find_character(unsigned char test_char[8][8], int *index_out, int *flip_out)
{
	int search_char;
	for(search_char = 0; search_char < MAXCHAR; search_char++)
	{
		//compare non-flipped
		int match_nonflipped = 1;
		int row;
		int col;
		for(row=0;row<8;row++)
		{
			for(col=0;col<8;col++)
			{
				if(test_char[row][col] != output_characters[search_char][row][col])
					match_nonflipped = 0;
			}
		}
		
		if(match_nonflipped)
		{
			*index_out = search_char;
			*flip_out = 0;
			return 1;
		}
		
		//compare hflip
		int match_hflip = 1;
		for(row=0;row<8;row++)
		{
			for(col=0;col<8;col++)
			{
				if(test_char[row][col] != output_characters[search_char][row][7-col])
					match_hflip = 0;
			}
		}
		
		if(match_hflip)
		{
			*index_out = search_char;
			*flip_out = 1;
			return 1;
		}
		
		//compare vflip
		int match_vflip = 1;
		for(row=0;row<8;row++)
		{
			for(col=0;col<8;col++)
			{
				if(test_char[row][col] != output_characters[search_char][7-row][col])
					match_vflip = 0;
			}
		}
		
		if(match_vflip)
		{
			*index_out = search_char;
			*flip_out = 2;
			return 1;
		}
		
		//compare hvflip
		int match_hvflip = 1;
		for(row=0;row<8;row++)
		{
			for(col=0;col<8;col++)
			{
				if(test_char[row][col] != output_characters[search_char][7-row][7-col])
					match_hvflip = 0;
			}
		}
		
		if(match_hvflip)
		{
			*index_out = search_char;
			*flip_out = 3;
			return 1;
		}
		
	}
	
	return 0;
}

int main(int argc, const char **argv)
{
	//Read input data
	memset(output_characters, 0, sizeof(output_characters));
	memset(output_charmap, 0, sizeof(output_charmap));
	size_t bytes_read = fread(bg_raw, 1, MAP_HEIGHT_CH*MAP_WIDTH_CH*64, stdin);
	if(bytes_read != MAP_HEIGHT_CH*MAP_WIDTH_CH*64)
	{
		fprintf(stderr, "failed to read input\n");
		exit(-1);
	}
	
	int characters_used = 1; //allow blank char initially
	
	//Iterate over each of the characters
	int input_c_row;
	int input_c_col;
	for(input_c_row=0;input_c_row<MAP_HEIGHT_CH;input_c_row++)
	{
		for(input_c_col=0;input_c_col<MAP_WIDTH_CH;input_c_col++)
		{
			//isolate the character we want
			unsigned char bg_character[8][8];
			memcpy(&(bg_character[0][0]), &(bg_raw[ (input_c_row*8)+0 ][ input_c_col * 8 ]), 8);
			memcpy(&(bg_character[1][0]), &(bg_raw[ (input_c_row*8)+1 ][ input_c_col * 8 ]), 8);
			memcpy(&(bg_character[2][0]), &(bg_raw[ (input_c_row*8)+2 ][ input_c_col * 8 ]), 8);
			memcpy(&(bg_character[3][0]), &(bg_raw[ (input_c_row*8)+3 ][ input_c_col * 8 ]), 8);
			memcpy(&(bg_character[4][0]), &(bg_raw[ (input_c_row*8)+4 ][ input_c_col * 8 ]), 8);
			memcpy(&(bg_character[5][0]), &(bg_raw[ (input_c_row*8)+5 ][ input_c_col * 8 ]), 8);
			memcpy(&(bg_character[6][0]), &(bg_raw[ (input_c_row*8)+6 ][ input_c_col * 8 ]), 8);
			memcpy(&(bg_character[7][0]), &(bg_raw[ (input_c_row*8)+7 ][ input_c_col * 8 ]), 8);
			
			int position;
			int flip;
			if(find_character(bg_character, &position, &flip)) //Existing character
			{
				//Store the map entry
				output_charmap[input_c_row][input_c_col] = position;
				if(flip & 1) //hflip
					output_charmap[input_c_row][input_c_col] |= 0x0200;
				if(flip & 2) //vflip
					output_charmap[input_c_row][input_c_col] |= 0x0400;
				
				if(flip == 0)
					fprintf(stderr, "%3Xn ", position);
				if(flip == 1)
					fprintf(stderr, "%3Xh ", position);
				if(flip == 2)
					fprintf(stderr, "%3Xv ", position);
				if(flip == 3)
					fprintf(stderr, "%3Xb ", position);
			}
			else //Need to output a new one
			{
				if(characters_used >= MAXCHAR)
				{
					fprintf(stderr, "too many unique chars!\n");
					fprintf(stderr, "got to row %d col %d\n", input_c_row, input_c_col);
					exit(-1);
				}
				
				//Store the character
				memcpy(output_characters[characters_used], bg_character, 64);
								
				//Store the map entry
				output_charmap[input_c_row][input_c_col] = characters_used;
				
				fprintf(stderr, "%3Xn ", characters_used);
				
				characters_used++;
			}
			
		}
		
		fprintf(stderr, "\n");
		
	}
	
	//Spit out the character map, because it's fixed-size
	for(input_c_row=1;input_c_row<MAP_HEIGHT_CH;input_c_row++) //skip first row
	{
		for(input_c_col=0;input_c_col<MAP_WIDTH_CH;input_c_col++)
		{
			fputc( (output_charmap[input_c_row][input_c_col]     ) & 0xFF, stdout);
			fputc( (output_charmap[input_c_row][input_c_col] >> 8) & 0xFF, stdout);
		}
	}
	
	//Spit out characters, bitplane format
	int outchar;
	for(outchar=0;outchar<MAXCHAR;outchar++)
	{
		int char_row;
		for(char_row=0;char_row<8;char_row++)
		{
			//Each output byte contains 1 bit of each pixel
			unsigned char a = 0;
			unsigned char b = 0;
			unsigned char c = 0;
			unsigned char d = 0;
			
			int char_col;
			for(char_col=0;char_col<8;char_col++)
			{
				a = a<<1;
				b = b<<1;
				c = c<<1;
				d = d<<1;
				
				if(output_characters[outchar][char_row][char_col] & 0x01)
					a = a | 1;
				if(output_characters[outchar][char_row][char_col] & 0x02)
					b = b | 1;
				if(output_characters[outchar][char_row][char_col] & 0x04)
					c = c | 1;
				if(output_characters[outchar][char_row][char_col] & 0x08)
					d = d | 1;
			}
			
			fputc( a, stdout );
			fputc( b, stdout );
			fputc( c, stdout );
			fputc( d, stdout );
		}
	}
	
	return 0;
}