//generate_spr.c
//Takes a raw image and makes a set of sprite characters
//Bryan Topp <betopp@betopp.com> 2015

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define IMAGE_WIDTH_CH 32
unsigned char bg_raw[8][IMAGE_WIDTH_CH*8];
unsigned char output_characters[IMAGE_WIDTH_CH][8][8];

int main(int argc, const char **argv)
{
	//Read input data
	memset(output_characters, 0, sizeof(output_characters));
	size_t bytes_read = fread(bg_raw, 1, IMAGE_WIDTH_CH*64, stdin);
	if(bytes_read != IMAGE_WIDTH_CH*64)
	{
		fprintf(stderr, "failed to read input\n");
		exit(-1);
	}
	
	int characters_used = 0;
	
	//Iterate over each of the characters
	int input_c_col;
	for(input_c_col=0;input_c_col<IMAGE_WIDTH_CH;input_c_col++)
	{
		//isolate the character we want
		unsigned char bg_character[8][8];
		memcpy(&(bg_character[0][0]), &(bg_raw[ 0 ][ input_c_col * 8 ]), 8);
		memcpy(&(bg_character[1][0]), &(bg_raw[ 1 ][ input_c_col * 8 ]), 8);
		memcpy(&(bg_character[2][0]), &(bg_raw[ 2 ][ input_c_col * 8 ]), 8);
		memcpy(&(bg_character[3][0]), &(bg_raw[ 3 ][ input_c_col * 8 ]), 8);
		memcpy(&(bg_character[4][0]), &(bg_raw[ 4 ][ input_c_col * 8 ]), 8);
		memcpy(&(bg_character[5][0]), &(bg_raw[ 5 ][ input_c_col * 8 ]), 8);
		memcpy(&(bg_character[6][0]), &(bg_raw[ 6 ][ input_c_col * 8 ]), 8);
		memcpy(&(bg_character[7][0]), &(bg_raw[ 7 ][ input_c_col * 8 ]), 8);

		//Store the character
		memcpy(output_characters[input_c_col], bg_character, 64);		
	}
		
	//Spit out characters, bitplane format
	int outchar;
	for(outchar=0;outchar<IMAGE_WIDTH_CH;outchar++)
	{
		int char_row;
		for(char_row=0;char_row<8;char_row++)
		{
			//Each output byte contains 1 bit of each pixel
			unsigned char a = 0;
			unsigned char b = 0;
			unsigned char c = 0;
			unsigned char d = 0;
			
			int char_col;
			for(char_col=0;char_col<8;char_col++)
			{
				a = a<<1;
				b = b<<1;
				c = c<<1;
				d = d<<1;
				
				if(output_characters[outchar][char_row][char_col] & 0x01)
					a = a | 1;
				if(output_characters[outchar][char_row][char_col] & 0x02)
					b = b | 1;
				if(output_characters[outchar][char_row][char_col] & 0x04)
					c = c | 1;
				if(output_characters[outchar][char_row][char_col] & 0x08)
					d = d | 1;
			}
			
			fputc( a, stdout );
			fputc( b, stdout );
			fputc( c, stdout );
			fputc( d, stdout );
		}
	}
	
	return 0;
}