;bricky.s
;SMS Brick-breaking game

;Homebrew release tagging
.SDSCTAG 0.1, "SMS Bricky", "Brick-breaking game for SMS", "Bryan E. Topp"

;SMS library functions
.INCLUDE "src/sms_mem.i"
.INCLUDE "src/sms_vdp.i"
.INCLUDE "src/sms_psg.i"

;Game control variables
.RAMSECTION "variables" SLOT 3
	score_bcd:	DSB 3    ;Current score, 6 digits (ones is always 0)
	topscore_bcd:	DSB 3    ;High score shown, 6 digits (ones is always 0)

	pause_flag:	DB       ;Toggled between 0 and ~0 by NMI (pause button)
	next_stage_bcd:	DB       ;Next stage to be set up when we finish this one
	frame_count:	DB       ;How many vblank interrupts we've caught (spin on this!)
	
	pad1_input:	DB       ;Cached pad data
	
	bricks_remain:  DW       ;How many bricks until stage ends
	ball_wait:      DB       ;If the ball is waiting to be deployed
	gutter_wait:    DB       ;If the ball is waiting to respawn, and if so, how long (countdown)
	spare_balls:    DB       ;How many extra lives the player has
	
	paddle_x:	DB       ;Position of player
	paddle_tilt:    DB       ;Which direction the paddle tilts (-1, 0, 1)
	
	ball_x:         DB       ;Position of ball (horizontal)
	ball_x_subpx:   DB       ;Fractional position of ball (horizontal)
	
	ball_x_dir:     DB       ;Direction of ball (horizontal)
	ball_x_speed:   DB       ;Fractional velocity of ball (horizontal)
	ball_x_super:   DB       ;Integer velocity of ball (horizontal)
	
	ball_y:         DB       ;Position of ball (vertical)
	ball_y_dir:     DB       ;Direction of ball (vertical) (always 1 px/frame)
	
	bounce_flag:    DB       ;Temporary used when the ball hits something (skips further collision checks)
	
	sound_count:    DB       ;Ticks until sound gets muted
	
	playfield:	DSB 1536 ;Mirror of VDP screen map, for collision checking
.ENDS

;VDP character layout - from background "theme" map
.DEFINE VDP_CHAR_BRICK_SINGLE 1 ; 1 = []
.DEFINE VDP_CHAR_BRICK_LEFT   2 ; 2 = [=
.DEFINE VDP_CHAR_BRICK_RIGHT  3 ; 3 = =]
.DEFINE VDP_CHAR_BRICK_FILL   4 ; 4 = ==

.DEFINE VDP_CHAR_ALT1_OFFSET 4
.DEFINE VDP_CHAR_ALT2_OFFSET 8

.DEFINE VDP_CHAR_NUMERAL_BASE 16

;VDP character layout - from sprite map
.DEFINE VDP_CHAR_SPRITE_BASE       	256 ;We use the second "page" of sprites - implicitly, 256 bit is set
.DEFINE VDP_CHAR_SPRITE_IX        	400 ;We load sprites starting at actual character index 400
.DEFINE VDP_CHAR_SPRITE_FIRST      	VDP_CHAR_SPRITE_IX-VDP_CHAR_SPRITE_BASE ;This is the first sprite as referenced
.DEFINE VDP_CHAR_SPRITE_PADDLE     	VDP_CHAR_SPRITE_FIRST+0
.DEFINE VDP_CHAR_SPRITE_BALL0      	VDP_CHAR_SPRITE_FIRST+4
.DEFINE VDP_CHAR_SPRITE_BALL1      	VDP_CHAR_SPRITE_FIRST+5
.DEFINE VDP_CHAR_SPRITE_STAR_HOLLOW	VDP_CHAR_SPRITE_FIRST+6
.DEFINE VDP_CHAR_SPRITE_STAR_FILLED	VDP_CHAR_SPRITE_FIRST+7
.DEFINE VDP_CHAR_SPRITE_ROCK       	VDP_CHAR_SPRITE_FIRST+8
.DEFINE VDP_CHAR_SPRITE_BLAST_R    	VDP_CHAR_SPRITE_FIRST+12
.DEFINE VDP_CHAR_SPRITE_BLAST_D    	VDP_CHAR_SPRITE_FIRST+16
.DEFINE VDP_CHAR_SPRITE_BLAST_L    	VDP_CHAR_SPRITE_FIRST+20
.DEFINE VDP_CHAR_SPRITE_POWERUP    	VDP_CHAR_SPRITE_FIRST+24


;Control bits (bit vector from joystick input port)
.DEFINE JOY_BIT_U 0
.DEFINE JOY_BIT_D 1
.DEFINE JOY_BIT_L 2
.DEFINE JOY_BIT_R 3
.DEFINE JOY_BIT_T1 4
.DEFINE JOY_BIT_T2 5

;Z80 code vectors
.ORG $0000
	jp reset_handler
.ORG $0038
	jp int_handler
.ORG $0066
	jp nmi_handler


;Main program code - WLA may relocate within bank 0 / slot 0
.BANK 0 SLOT 0
.SECTION "text" ALIGN 2 FREE

	;------------------------------------------------;
	;Macros to play a beep
	;------------------------------------------------;
	
	.MACRO beep_a ;Used for brick breaking
		;kill old sound
		psg_silence
		
		;set channels
		psg_channel_set $0 %1011011000 %0111
		psg_noise_set %110 %0011

		;set timeout to stop beep
		ld A, 6
		ld (sound_count), A
	.ENDM
	
	.MACRO beep_b ;Used for paddle ping
	
		psg_silence
		
		psg_channel_set $1 %0101101100 %0010
		psg_channel_set $2 %0010110110 %0110
		
		

		ld A, 6
		ld (sound_count), A
		
	.ENDM
	
	.MACRO beep_c ;Used for wall/ceiling ping

		psg_silence
		
		psg_channel_set $1 %1011011000 %010
		psg_channel_set $2 %0101101100 %110

		ld A, 6
		ld (sound_count), A
		
	.ENDM

	;================================================;
	

	;------------------------------------------------;
	;Macro to read gamepads
	;------------------------------------------------;
	.MACRO poll_pad
		in A, ($DC)
		ld (pad1_input), A
	.ENDM
	;================================================;


	;------------------------------------------------;
	;Saves the top score across resets in the shadow registers
	;------------------------------------------------;
	preserve_topscore:
		exx
		ld A, (topscore_bcd)
		ld B, A
		
		ld A, (topscore_bcd+1)
		ld C, A
		
		ld A, (topscore_bcd+2)
		ld D, A
		
		ld A, $BE
		ld H, A
		
		ld A, $EF
		ld L, A
		exx
	ret
	;================================================;

	;------------------------------------------------;
	;Entry point on system reset
	;------------------------------------------------;
	reset_handler:
		
		;Kill the display during setup
		vdp_set_register 1, %10000000
		
		;Turn off sound
		psg_silence
	
		;Z80 interrupt mode 1; interrupts disabled
		;We don't keep interrupts on during main execution.
		;Interrupts are enabled when we want to wait for vblank, and then killed afterwards.
		di
		im 1
		
		;Set the stack pointer to almost the top of RAM
		ld SP, $DFF0
		
		;Zero out (almost) all the RAM
		ld A, 0
		ld ($C000), A ;put a 0 in the first byte of RAM
		ld HL, $C000  ;copy that 0 to the next 8192-16 bytes (don't mash the cartridge memory mapping)
		ld DE, $C001
		ld BC, $1FF0
		ldir
		
		;Zero out all the VRAM
		ld HL, $C000 ;Source data from our zeroed RAM
		vdp_write_vram_256 0, 64 ;Writes 64 blocks of 256 bytes to VRAM address 0
		
		
		;Set the direction bits for the controller port so we can support MD pads
		ld A, %10100101 ;Pin TH is an output, and held high for the MD mux, on both ports
		out ($3F), A
		
		;Make sure banking is set to title screen
		ld A, :title
		ld (SLOT_2_REGISTER), A
		
		;Initialize the screen map - beginning of title background blob
		ld HL, title
		vdp_write_vram_256 VDP_SCREEN_MAP_LOC, 6
		
		;Increment HL pointer past the map, into the tile graphics section of the title blob
		ld HL, title
		ld DE, 1536
		add HL, DE
		vdp_write_vram_256 $0000, 50 ;400 characters * (32 bytes / 1 character) * (1 block-write / 256 bytes)
		
		
		;Initialize palettes
		ld HL, title_pal
		vdp_load_palette_bg
		
		ld HL, sprites_pal
		vdp_load_palette_fg
		
		
		;Initialize the default top score
		ld A, $00
		ld (topscore_bcd+0), A
		ld A, $01
		ld (topscore_bcd+1), A
		ld A, $00
		ld (topscore_bcd+2), A
		
		;Check if we have a top score preserved in the shadow registers
		exx
		ld A, H
		cp $BE
		jr NZ, _nograbscore
		ld A, L
		cp $EF
		jr NZ, _nograbscore
		
		_grabscore:
			ld A, B
			ld (topscore_bcd+0), A
			ld A, C
			ld (topscore_bcd+1), A
			ld A, D
			ld (topscore_bcd+2), A
		_nograbscore:
		exx

		
		
		
		;Load the VDP registers (set register 1 last, because it will unblank display)
		vdp_set_register 0, %10010110 ; VSI | HSI | LCB | IE1 | SS | 1  |  1 | ES
		vdp_set_register 2, $FF       ; Screen map base addr = $3800
		vdp_set_register 3, $FF       ; "R3 should always be set to $FF"
		vdp_set_register 4, $FF       ; "R4 should always be set to $FF"
		vdp_set_register 5, $FF       ; Sprite table base addr = $3F00
		vdp_set_register 6, $FF       ; Sprite characters at end of VRAM
		vdp_set_register 7, $F0       ; Border color = first entry of second palette
		vdp_set_register 8, $00       ; H scroll amount = 0
		vdp_set_register 9, $00       ; V scroll amount = 0
		vdp_set_register 10, $FF      ; No hblank interrupts
		vdp_set_register 1, %11100000 ; 1   | DIS |  IE |   0 |  0 | 0  | SZ | 0
		
	
		;Wait for the user to press start
		start_wait:
			vdp_wait_vbl
			poll_pad
			ld A, (pad1_input)
			bit JOY_BIT_T1, A
		jr NZ, start_wait
		
	
		;Start a game on stage 1
		ld A, $01
		ld (next_stage_bcd), A

		;Start a game with 3 spare balls
		ld A, 3
		ld (spare_balls), A

		call run_game
		
	rst 0 ;Reset if game finishes
	;================================================;


	;------------------------------------------------;	
	;Entry point on maskable interrupt (video timers)
	;------------------------------------------------;
	int_handler:
		exx
		ex AF, AF'
		;Read the type of interrupt from the VDP, clearing it
		in A, (VDP_COMMAND_PORT) 

		;Reset the system if the reset button is pressed.
		in A, ($DD)
		bit 4, A
		jr NZ, no_reset
			exx
			ex AF, AF'
			call preserve_topscore
			rst 0
		no_reset:
		
		;Increment the frame counter (stuff waits for this)
		ld HL, frame_count
		inc (HL)
		
		;Decrement the sound counter
		ld A, (sound_count)
		dec A
		ld (sound_count), A
		cp 0
		jr NZ, +
			;Turn off PSG if sound count is 0
			psg_silence
		+:
		
		exx
		ex AF, AF'
	reti ;Return from maskable interrupt
	;================================================;	


	;------------------------------------------------;
	;Entry point on nonmaskable interrupt (pause button)
	;------------------------------------------------;
	nmi_handler:
		ex AF, AF'
		
		;Toggle the pause flag between 0 and ~0
		ld A, (pause_flag)
		cpl
		ld (pause_flag), A

		ex AF, AF'
	retn ;Return from nonmaskable interrupt
	;================================================;


	;------------------------------------------------;
	;Main loop for game program
	;------------------------------------------------;
	run_game:

		;Initialize the stage
		call game_stage_init		

		;Loop over game frames
		game_tick_loop:
			poll_pad

			;Run the game tick
			call game_update

			;Wait for blanking
			vdp_wait_vbl

			;Push new data to VDP
			call update_scores
			call update_sprites

		;Continue to run game ticks while there are bricks left
		ld A, (bricks_remain)
		cp 0
		jr NZ, game_tick_loop
		ld A, (bricks_remain+1)
		cp 0
		jr NZ, game_tick_loop
	
	
		;Increment the next-stage counter
		ld HL, next_stage_bcd
		inc (HL)
		
		;Run the next stage
		jr run_game
		;Todo: return at game-over
	ret
	;================================================;


	;------------------------------------------------;
	;Macro to copy line from the pattern table to the playfield.
	;Zeroes the high bytes of the playfield entries.
	;Leaves gaps where the pattern table has 0s.
	;Expects address of playfield location in DE
	;Expects address of pattern table row in HL
	;------------------------------------------------;
	.MACRO patternoverlay
		;Must preserve IX, IY, B
		ld C, 22
		-:
			;if there's a space in the pattern table, don't overwrite
			ld A, 0
			cp (HL)
			jr Z, +
				;count up bricks remaining
				ld A, (bricks_remain)
				add A, 1
				ld (bricks_remain), A
				ld A, (bricks_remain+1)
				adc A, 0
				ld (bricks_remain+1), A
				ld A, 0
			
				;pattern table had an entry -> copy it
				ldi ;Loads the pattern table entry into the low byte of the playfield entry.
				
				ld (DE), A ;A=0. Covers the high byte of the playfield entries.
				inc DE
				jr ++
			+:
				;pattern table was 0 -> just advance pointers
				dec BC
				inc DE
				inc DE
				inc HL
			++:
		cp C
		jr NZ, -
		ld A, 0
	.ENDM
	;================================================;

	
	;------------------------------------------------;
	;Ding ding ding!
	;------------------------------------------------;
	.MACRO awardpoint
		beep_a
	
		or 0 ;Clears flags
	
		ld A, (score_bcd+2)
		inc A
		daa
		ld (score_bcd+2), A
		
		ld A, (score_bcd+1)
		adc A, 0
		daa
		ld (score_bcd+1), A
		
		ld A, (score_bcd+0)
		adc A, 0
		daa
		ld (score_bcd+0), A
		
		;Compare from high byte downwards - keep comparing if high score = new score. set high score if lower than new score.
		ld A, (topscore_bcd+0)
		ld B, A
		ld A, (score_bcd+0)
		cp B
		jr C, ++ ;High score was larger, don't update it
		jr NZ, + ;High score was lower in this digit - definitely update it
		
		ld A, (topscore_bcd+1)
		ld B, A
		ld A, (score_bcd+1)
		cp B
		jr C, ++
		jr NZ, +
		
		ld A, (topscore_bcd+2)
		ld B, A
		ld A, (score_bcd+2)
		cp B
		jr C, ++
		jr NZ, +
			
		+:
			ld A, (score_bcd+2)
			ld (topscore_bcd+2), A
			ld A, (score_bcd+1)
			ld (topscore_bcd+1), A
			ld A, (score_bcd+0)
			ld (topscore_bcd+0), A
		
		++:
		
	.ENDM
	;================================================;


	;------------------------------------------------;
	;Game setup function for stage init, called once per stage transition
	;------------------------------------------------;
	game_stage_init:
	
		;Wait for vblank, then turn the display off
		vdp_wait_vbl
		vdp_set_register 1, %10100000 ; 1   | DIS |  IE |   0 |  0 | 0  | SZ | 0
		
		;Select the correct background theme.
		ld A, (next_stage_bcd)
		and $03
		cp 0
		jr NZ, +
			ld A, :bg3
			ld (SLOT_2_REGISTER), A
			jr ++
		+:
		cp 1
		jr NZ, +
			ld A, :bg0
			ld (SLOT_2_REGISTER), A
			jr ++
		+:
		cp 2
		jr NZ, +
			ld A, :bg2
			ld (SLOT_2_REGISTER), A
			jr ++
		+:
			ld A, :bg1
			ld (SLOT_2_REGISTER), A
		++:
		
		;Copy tile graphics from background blob into VRAM
		ld HL, bg0
		ld DE, 1536
		add HL, DE
		vdp_write_vram_256 $0000, 50 ;400 characters * (32 bytes / 1 character) * (1 block-write / 256 bytes)

		;Copy screen map into playfield, so we don't have to read stuff back from VDP when we do collision checks
		ld DE, playfield
		ld HL, bg0
		ld B, 6 ;1536 bytes
		ld C, 0 
		ldir
		
		;Zero the brick count
		ld A, 0
		ld (bricks_remain), A
		ld (bricks_remain+1), A
		

		;Find the location in the stage table for this stage
		ld A, (next_stage_bcd) ;Pull tens digit from stage and set aside
		srl A
		srl A
		srl A
		srl A
		ld B, A
		
		ld A, (next_stage_bcd) ;Pull ones digit from stage
		and $0F
		
		.REPEAT 10 ;Add the tens digit back in, 10 times
			add A, B 
		.ENDR
		
		;Accumulator now contains stage number.
		;Only have 16 stage patterns
		and $0F
		
		;Paint brick patterns into the playfield in RAM
		ld IX, stage_table ;Start with the stage table base address	
		
		;Add the stage offset
		ld B, 0
		ld C, A
		.REPEAT 16
			add IX, BC
		.ENDR
		
		
		ld IY, playfield+66
		ld B, 16 ;16 rows
		stage_init_row_loop:
			;Read the row number from the stage table, and multiply by 32
			ld D, 0
			ld E, (IX+0)
			sla E
			rl D
			sla E
			rl D
			sla E
			rl D
			sla E
			rl D
			sla E
			rl D
			
			;Use row*32 as offset into pattern table - result in HL
			ld HL, pattern_table
			add HL, DE
			
			;Load DE with position in playfield
			push IY
			pop DE
			
			patternoverlay ;doesn't clobber IX, IY, or B
			
			;Advance position in playfield
			ld D, 0
			ld E, 64
			add IY, DE
			
			;Advance position in stage table
			inc IX
		djnz stage_init_row_loop
		
		;Copy that into the VDP screen map
		ld HL, playfield
		vdp_write_vram_256 VDP_SCREEN_MAP_LOC, 6		
		
		;Copy sprite graphics into VRAM
		ld HL, sprites
		vdp_write_vram_256 VDP_CHAR_SPRITE_IX*32, 4 ;32 characters * (32 bytes / 1 character) * (1 block-write / 256 bytes)
		
		;Initialize palettes
		ld HL, bg0_pal
		vdp_load_palette_bg
		
		ld HL, sprites_pal
		vdp_load_palette_fg
		
		;Position game elements
		
		ld A, 80
		ld (paddle_x), A
		add A, 12
		ld (ball_x), A
		
		ld A, 152
		ld (ball_y), A
		
		ld A, 0
		ld (ball_x_subpx), A
		
		ld A, 0
		ld (ball_x_speed), A
		ld A, 0
		ld (ball_x_super), A
		
		ld A, 1
		ld (ball_y_dir), A
		ld A, -1
		ld (ball_x_dir), A
		
		
		;Set up status of ball
		ld A, 1
		ld (ball_wait), A
		
		ld A, 0
		ld (gutter_wait), A
		
		ld A, 0
		ld (pause_flag), A
		
		
	
		
		;Load "foreground" elements
		call update_scores
		call update_sprites
	
		;Turn the display back on
		vdp_set_register 1, %11100000 ; 1   | DIS |  IE |   0 |  0 | 0  | SZ | 0
	
	ret
	;================================================;


	;------------------------------------------------;
	;Runs the game for one tick.
	;------------------------------------------------;
	game_update:
	
		;Just return if we're paused
		ld A, (pause_flag)
		cp $FF
		jr NZ, +
			ret
		+:
	
		;Start from 0 tilt each frame
		ld A, 0
		ld (paddle_tilt), A
	
		;Check if the player should be moving left or right
		ld HL, paddle_x
		ld A, (pad1_input)
		bit JOY_BIT_L, A
		jr NZ, +
			dec (HL)
			dec (HL)
			
			ld A, (paddle_tilt)
			dec A
			ld (paddle_tilt), A
		+:
		bit JOY_BIT_R, A
		jr NZ, +
			inc (HL)
			inc (HL)
			
			ld A, (paddle_tilt)
			inc A
			ld (paddle_tilt), A
		+:
		
		;Cheat
		;ld A, (ball_x)
		;ld (paddle_x), A
		
		;Cap their position at the edges of the stage
		ld A, (paddle_x)
		cp 8
		jr NC, +
			ld A, 8
		+:
		cp 152
		jr C, +
			ld A, 152
		+:
		ld (paddle_x), A
		
		;If the ball is in the gutter, wait for it and do nothing else
		ld A, (gutter_wait)
		cp 0
		jr Z, +
			call gutter_waiting
			ret
		+:
		
		;If the ball is not deployed, just stick it to the paddle
		ld A, (ball_wait)
		cp 0
		jr NZ, +
			;Ball deployed, move
		
			;Clear the bounce flag
			ld A, 0
			ld (bounce_flag), A
			
			;Ball motion
			call ball_update_v
			call ball_update_h
			
			ret
		+:
			;Stick the ball to the paddle and wait
			ld A, (paddle_x)
			add A, 12
			ld (ball_x), A
			
			ld A, 152
			ld (ball_y), A
			
			;Check if we should still not deploy
			ld A, (pad1_input)
			bit JOY_BIT_T1, A
			;cp 255 ;Cheat
			jr NZ, +
				;Trigger pressed - deploy
				ld A, 0
				ld (ball_wait), A
				
				ld A, -1
				ld (ball_y_dir), A
				
				ld A, 1
				ld (ball_x_dir), A
				
				ld A, 0
				ld (ball_x_subpx), A
				
				ld A, 160
				ld (ball_x_speed), A
				ld A, 0
				ld (ball_x_super), A
			+:
			ret
	ret
	;================================================;	
	
	
	;------------------------------------------------;
	;Waits for ball to respawn.
	;------------------------------------------------;
	gutter_waiting:
		ld A, (gutter_wait)
		dec A
		ld (gutter_wait), A

		;Don't do anything if the timer isn't done
		ld A, (gutter_wait)
		cp 0
		jr Z, +
			ret
		+:

		;Game over if no spare balls
		ld A, (spare_balls)
		cp 0
		jr NZ, +
			call do_gameover ;Never actually returns
			ret
		+:

		dec A
		ld (spare_balls), A

		ld A, 1
		ld (ball_wait), A

	ret
	;================================================;
	
	
	;------------------------------------------------;
	;Moves the ball vertically - decides up/down
	;------------------------------------------------;
	ball_update_v:
		
		;Is the ball moving down?
		ld A, (ball_y_dir)
		cp 1
		jr NZ, +
			call ball_update_v_down
			ret
		+:
		
		
		;Is the ball moving up?
		ld A, (ball_y_dir)
		cp -1
		jr NZ, +
			call ball_update_v_up
			ret
		+:	
	ret
	;================================================;
	
	
	;------------------------------------------------;
	;Moves the ball vertically downwards (+)
	;------------------------------------------------;	
	ball_update_v_down:
		
		;Increment position
		ld A, (ball_y)
		inc A
		ld (ball_y), A
		
		;Check the bottom of the ball for collision with bricks
		add A, 6
		call collide_v
				
		;Then work from the bottom-up for fixed collision points:
				
		;All the way at the bottom of the screen - into the gutter
		ld A, (ball_y)
		cp 192
		jr C, +
			ld A, 80
			ld (gutter_wait), A
			ret
		+:
	
		;we're not quite at the bottom of the screen - are we below the top of the paddle?
		ld A, (ball_y)
		cp 155 ;allow some leniency
		jr C, +
			ret ;Nothing else we can collide with vertically
		+:
				
		
		;we're above the top of the paddle - did we just hit it?
		ld A, (ball_y)
		cp 153 
		jr C, +
		
			;If we're beneath the point of colliding with the paddle, check if we hit or miss it
			ld A, (ball_x)
			ld B, A
			
			ld A, (paddle_x)
			add A, -8
			
			sub B
			
			jr NC, + ;left of the paddle - no collision
			
			
			ld A, (paddle_x)
			add A, 31
			
			sub B
			
			jr C, + ;right of paddle - no collision
			
			
			beep_b
			
			ld A, (ball_y_dir) ;invert the direction
			neg
			ld (ball_y_dir), A
			
			;Paddle can spin the ball
			ld A, (paddle_tilt)
			cp -1
			jr NZ, _notleft
				call add_spin_left
			_notleft:
			
			ld A, (paddle_tilt)
			cp 1
			jr NZ, _notright
				call add_spin_right
			_notright:
			
		+:
		
	ret
	;================================================;


	;------------------------------------------------;
	;Slightly changes the ball's horizontal velocity
	;------------------------------------------------;	
	add_spin_left:
	
	
		ld A, (ball_x_dir)
		cp 1
		jr NZ, +
			call decrease_ball_vel			
			ret
		+:
			call increase_ball_leftward_vel
			ret
		
	ret
	;================================================;
	
	
	;------------------------------------------------;
	;Slightly changes the ball's horizontal velocity
	;------------------------------------------------;	
	add_spin_right:
		ld A, (ball_x_dir)
		cp 1
		jr NZ, +
			call increase_ball_rightward_vel			
			ret
		+:
			call decrease_ball_vel
			ret
	ret
	;================================================;	
	
	
	;------------------------------------------------;
	;Helpers for the above
	;------------------------------------------------;
	decrease_ball_vel:
		;Easy case if subpixel velocity is greater than 64.
		ld A, (ball_x_speed)
		sub 64
		ld (ball_x_speed), A 
		jr C, +
			;Carry flag not set - X subpixel speed > 64.
			;Just change subpixel speed and return
			ret
		+:
		
		;Need to decrease whole pixel velocity as well
		
		;Decreasing across 0 is actually a change of direction
		ld A, (ball_x_super)
		cp 0
		jr NZ, +
			;Whole pixel velocity was 0
			;Reverse direction
			ld A, (ball_x_dir)
			neg
			ld (ball_x_dir), A
			
			;Negate subpixel velocity
			ld A, (ball_x_speed)
			neg
			ld (ball_x_speed), A
			
			ret
		+:
		
		;Regular old whole-pixel velocity decrease
		ld A, (ball_x_dir)
		cp 1
		jr NZ, +
			ld A, (ball_x_super)
			dec A
			ld (ball_x_super), A
			
			ret
		+:
			ld A, (ball_x_super)
			inc A
			ld (ball_x_super), A
			
			ret
	ret
	
	
	increase_ball_rightward_vel:
		
		;Easy one - everything is positive
		ld A, (ball_x_speed)
		add A, 64
		ld (ball_x_speed), A
		ld A, (ball_x_super)
		adc A, 0
		ld (ball_x_super), A
		
	ret

	
	increase_ball_leftward_vel:
		
		;Increase subpixel velocity, but possibly use that to decrement whole-pixel velocity
		ld A, (ball_x_speed)
		add A, 64
		ld (ball_x_speed), A
		
		jr NC, +
			ld A, (ball_x_super)
			dec A
			ld (ball_x_super), A
		+:
	ret
	;================================================;
	
	

	

	;------------------------------------------------;
	;Moves the ball vertically upwards (-)
	;------------------------------------------------;	
	ball_update_v_up:

		;Decrement position
		ld A, (ball_y)
		dec A
		ld (ball_y), A

		;Check slightly below the top of the ball for collision
		inc A

		;Collision subroutine uses the "A" we specify as the vertical position to check
		call collide_v

		;bounce off the top of the screen (are we way off the botttom from underflow?)
		ld A, (ball_y)
		cp 200
		jr C, +
			beep_c
		
			ld A, (ball_y_dir) ;invert the direction
			neg
			ld (ball_y_dir), A

			ld A, 0 ;cap the position
		+:

		ld (ball_y), A
	ret
	;================================================;


	;------------------------------------------------;
	;Performs collision checks with top/bottom of brick
	;Expects A to contain Y position of ball (i.e. could be top of ball or bottom of ball)
	;------------------------------------------------;
	collide_v:
		;Each row of the tilemap is 64 bytes long, and is 8 pixels tall.
		;Zero the low 3 bits of the ball position, then shift left 3
		;(So we have the position/8 followed by 6 zeroes - a row addr)
		and $F8
		ld E, A
		ld D, 0
		sla E
		rl D
		sla E
		rl D
		sla E
		rl D
		
		;Apply the row offset to the playfield
		ld HL, playfield
		add HL, DE
		
		;Find the block offset from X position
		;We want to divide position by 8 and then multiply by 2 (2 bytes/block)
		;So we're losing 3 bits and gaining 1
		;Zero out the low 3 bits, shift right twice.
		ld A, (ball_x)
		
		and $F8
		srl A
		srl A

		;Apply the block offset to the row address
		ld E, A
		ld D, 0
		add HL, DE
		
		;Multiple possibilities now:
		;1. The ball is exactly aligned on an 8px boundary. Only check the address as computed. (x%8 == 0)
		;2. The ball is overlapping the next character as well. Check the address and the next. (x%8 == 1, 2, 3)
		;3. The ball is primarily on the next character. Check the next address, then this one. (x%8 == 4, 5, 6)
		;4. The ball is totally on the next character. (x%8 == 7)
		ld A, (ball_x)
		
		and $07
		cp 0
		jr NZ, +
			;exact X alignment - check only this address
			call check_brick_v
			jr done_colliding
		+:
		cp 4
		jr NC, +
			;overlapping the next brick slightly - check this one first
			push HL
			call check_brick_v
			pop HL
			
			;are we still moving up?
			ld A, (bounce_flag)
			cp 0
			jr NZ, done_colliding
			
			;need to do the second check if the first one didn't turn us around
			inc HL
			inc HL
			call check_brick_v
			jr done_colliding
		+:
		cp 7
		jr NC, +
		
			;overlapping the next brick in a major way - check the other first
			push HL
			inc HL
			inc HL
			call check_brick_v
			pop HL
			
			;are we still moving up?
			ld A, (bounce_flag)
			cp 0
			jr NZ, done_colliding
			
			call check_brick_v
			jr done_colliding
		+:
			;only on the next brick
			inc HL
			inc HL
			call check_brick_v
			jr done_colliding

		done_colliding:
	
	ret
	;================================================;
	
	
	;------------------------------------------------;
	;Tests if a brick should stop the ball (vertical)
	;Expects HL to contain the address in playfield
	;------------------------------------------------;
	check_brick_v:
		;Check the high byte first - should be all 0 for bricks
		inc HL
		ld A, (HL)
		cp 0
		jr NZ, +
		
		;Then look at the low byte and check for brick character
		dec HL
		;Look up the brick that we hit
		ld A, (HL)
		cp 0
		jr Z, +	
		cp 16
		jr NC, +
			ld A, (ball_y_dir) ;invert the direction
			neg
			ld (ball_y_dir), A
			ld B, A
			
			;mark the bounce flag to skip further checks
			ld A, 1
			ld (bounce_flag), A
			
			ld A, (ball_y) ;pop out
			add A, B
			ld (ball_y), A
			
			;pop the brick
			ld D, H
			ld E, L
			
			call break_brick
		+:
	ret
	;================================================;
	
	
	;------------------------------------------------;
	;Moves the ball horizontally - decides L/R
	;------------------------------------------------;
	ball_update_h:

		;Ball moving right?
		ld A, (ball_x_dir)
		cp 1
		jr NZ, +		
			;subpixel accumulate
			ld A, (ball_x_speed)
			ld B, A
			ld A, (ball_x_subpx)
			add A, B
			ld (ball_x_subpx), A
			jr NC, + ;Didn't move a whole pixel yet
			
			;whole-pixel increment
			ld HL, ball_x
			inc (HL)
			
		+:
		
		;Ball moving left?
		ld A, (ball_x_dir)
		cp -1
		jr NZ, +
			;subpixel decumulate
			ld A, (ball_x_speed)
			ld B, A
			ld A, (ball_x_subpx)
			add A, B
			ld (ball_x_subpx), A
			jr NC, + ;Didn't move a whole pixel yet
			
			;whole-pixel decrement
			ld HL, ball_x
			dec (HL)
		
		+:
		
		;whole-pixel motion
		ld A, (ball_x_super)
		ld B, A
		ld A, (ball_x)
		add A, B
		ld (ball_x), A
	
	
		;check for bouncing on the side of the paddle
		call ball_collide_paddle_sides

		;check for block collision
		call collide_h
		
		;If the ball hit an edge, bounce it
		ld A, (ball_x)
		cp 8
		jr NC, +
			beep_c
		
			ld A, 8 ;cap position
			ld (ball_x), A
			
			ld A, 1 ;reverse subpixel direction
			ld (ball_x_dir), A
			
			ld A, (ball_x_super) ;reverse whole-pixel speed
			neg
			ld (ball_x_super), A
		+:
		
		ld A, (ball_x)
		cp 176
		jr C, +
			beep_c
		
			ld A, 176 ;cap position
			ld (ball_x), A
			
			ld A, -1 ;reverse subpixel direction
			ld (ball_x_dir), A

			ld A, (ball_x_super) ;reverse whole-pixel speed
			neg
			ld (ball_x_super), A
		+:
		
				
	ret
	;================================================;

	;------------------------------------------------;
	;Performs a collision check on the sides of a brick
	;------------------------------------------------;
	collide_h:
	
	
		;Return if we already bounced vertically
		ld A, (bounce_flag)
		cp 0
		jr Z, +
			ret
		+:	

		;Depending on the vertical direction of the ball, we check upper-left or lower-left corner
		ld A, (ball_y)
		inc A
		call collide_h_brick

		;Return if that succeeded
		ld A, (bounce_flag)
		cp 0
		jr Z, +
			ret
		+:

		;Check the next one down
		ld A, (ball_y)
		add A, 6
		call collide_h_brick

	ret
	;================================================;


	;------------------------------------------------;
	;Helper: check an individual brick on the left side
	;------------------------------------------------;
	collide_h_brick:
		;Each row of the tilemap is 64 bytes long, and is 8 pixels tall.
		;Zero the low 3 bits of the ball position, then shift left 3
		;(So we have the position/8 followed by 6 zeroes - a row addr)
		and $F8
		ld E, A
		ld D, 0
		sla E
		rl D
		sla E
		rl D
		sla E
		rl D

		;Apply the row offset to the playfield
		ld HL, playfield
		add HL, DE

		;Find the block offset from X position
		;We want to divide position by 8 and then multiply by 2 (2 bytes/block)
		;So we're losing 3 bits and gaining 1
		;Zero out the low 3 bits, shift right twice.

		;if we're moving right, bump the X coordinate right
		ld A, (ball_x_dir)
		cp -1
		ld A, (ball_x) ;Just use the plain (left) x coordinate
		jr Z, +
			add A, 7
		+:


		and $F8
		srl A
		srl A

		;Apply the block offset to the row address
		ld E, A
		ld D, 0
		add HL, DE


		;Check the high byte first - should be all 0 for bricks
		inc HL
		ld A, (HL)
		cp 0
		jr NZ, +

		;Then look at the low byte and check for brick character
		dec HL
		;Look up the brick that we hit
		ld A, (HL)
		cp 0
		jr Z, +	
		cp 16
		jr NC, +

			call ball_switch_h

			;mark the bounce flag to skip further checks
			ld A, 1
			ld (bounce_flag), A

			;ld A, (ball_x) ;pop out
			;add A, B
			;ld (ball_x), A

			;pop the brick
			ld D, H
			ld E, L

			call break_brick
		+:
	ret
	;================================================;


	;------------------------------------------------;
	;Handles if the ball hit the left or right edge of the paddle.
	;------------------------------------------------;
	ball_collide_paddle_sides:
		;If the ball is overlapping the paddle vertically, collide and bounce horizontally
		ld A, (ball_y)
		cp 168
		jr C, +
			ret ;Too far below paddle
		+:
		
		ld A, (ball_y)
		cp 154
		jr C, +
			;Below top of paddle, time to possibly bounce against the side
			
			ld A, (ball_x)
			ld B, A
			
			ld A, (paddle_x)
			add A, -7

			sub B

			jr NC, + ;left of the paddle - no collision

			ld A, (paddle_x)
			add A, 30

			sub B

			jr C, + ;right of paddle - no collision


			;Check which side of the paddle we're on
			ld A, (paddle_x)
			add A, 7
			sub B
			jr NC, _bumpleft ;left of center. ball should move left, 
				call ball_bump_paddle_rightside
				jr +
			_bumpleft:
				call ball_bump_paddle_leftside

		+:
	ret
	;================================================;


	;------------------------------------------------;
	;Forces ball to the left of the paddle, moving left
	;------------------------------------------------;
	ball_bump_paddle_leftside:
		ld A, (paddle_x)
		sub 7
		ld (ball_x), A
		
		call ball_switch_to_left
		call add_spin_left
		
	ret
	;================================================;
	
	;------------------------------------------------;
	;Forces ball to the right of the paddle, moving right
	;------------------------------------------------;
	ball_bump_paddle_rightside:
		ld A, (paddle_x)
		add A, 32
		ld (ball_x), A
		
		call ball_switch_to_right
		call add_spin_right
	ret
	;================================================;
	
	
	;------------------------------------------------;
	;Helpers to change X direction
	;------------------------------------------------;
	ball_switch_to_left:
		;check if we're already going left and don't change if so
		ld A, (ball_x_dir)
		cp -1
		jr Z, +
			;going right - switch directions to left
			ld A, -1 ;reverse subpixel direction
			ld (ball_x_dir), A

			ld A, (ball_x_super) ;reverse whole-pixel speed
			neg
			ld (ball_x_super), A
		+:
	ret
	
	ball_switch_to_right:
		;check if we're already going right and don't change if so
		ld A, (ball_x_dir)
		cp 1
		jr Z, +
			;going left - switch to right
			ld A, 1 ;reverse subpixel direction
			ld (ball_x_dir), A
			
			ld A, (ball_x_super) ;reverse whole-pixel speed
			neg
			ld (ball_x_super), A
			
		+:
	ret
	
	ball_switch_h:
		ld A, (ball_x_dir)
		cp -1
		jr Z, +
			call ball_switch_to_left
			ret
		+:
			call ball_switch_to_right
			ret
	ret
	;================================================;
	
	
	;------------------------------------------------;
	;Breaks the brick at the memory address specified
	;Expects DE to contain brick location
	;------------------------------------------------;
	break_brick:
		
		;DE will point to the low-byte (character number modulo 256) of a brick in the playfield
		;Make sure we're looking at the far left side of a brick (or a single-char brick)
		
		ld A, (DE)
		cp VDP_CHAR_BRICK_SINGLE
		jr Z, +
		cp VDP_CHAR_BRICK_LEFT
		jr Z, +
		cp VDP_CHAR_BRICK_SINGLE+VDP_CHAR_ALT1_OFFSET
		jr Z, +
		cp VDP_CHAR_BRICK_LEFT+VDP_CHAR_ALT1_OFFSET
		jr Z, +
		cp VDP_CHAR_BRICK_SINGLE+VDP_CHAR_ALT2_OFFSET
		jr Z, +
		cp VDP_CHAR_BRICK_LEFT+VDP_CHAR_ALT2_OFFSET
		jr Z, +
		
		;This isn't the beginning of a brick. Try again.
		dec DE
		dec DE ;each playfield entry is 2 bytes
		jr break_brick
		
		;This is the beginning of a brick. Award point and start clearing tiles.
		+:
		awardpoint
		
		;tile-clearing loop
		break_brick_inner:
		
		
		;Decrement bricks remaining
		ld A, (bricks_remain)
		sub 1
		ld (bricks_remain), A
		ld A, (bricks_remain+1)
		sbc A, 0
		ld (bricks_remain+1), A


		;Okay, it's a left-side or a single piece.
		;DE is somewhere within the playfield array.

		;Zero out those bytes in memory.
		ld H, D
		ld L, E
		ld A, 0
		ld (HL), A
		inc HL
		ld (HL), A

		;Find the location in VRAM of this playfield location
		ld H, D
		ld L, E
		ld BC, 65536+VDP_SCREEN_MAP_LOC-playfield
		add HL, BC

		;Ready the VDP to write there
		ld A, L
		out (VDP_COMMAND_PORT), A
		ld A, H
		or $40
		out (VDP_COMMAND_PORT), A


		;Now, find an offset in the BG array corresponding to the bricks we erased
		ld H, D
		ld L, E
		ld BC, bg0-playfield
		;Scoot HL over so it's now within the bg0 array rather than the playfield array
		add HL, BC
		
		ld C, VDP_DATA_PORT

		outi
		outi
		
		;Loop if this brick continues to the right
		inc DE
		inc DE
		ld A, (DE)
		cp VDP_CHAR_BRICK_FILL
		jr Z, +
		cp VDP_CHAR_BRICK_RIGHT
		jr Z, +
		cp VDP_CHAR_BRICK_FILL+VDP_CHAR_ALT1_OFFSET
		jr Z, +
		cp VDP_CHAR_BRICK_RIGHT+VDP_CHAR_ALT1_OFFSET
		jr Z, +
		cp VDP_CHAR_BRICK_FILL+VDP_CHAR_ALT2_OFFSET
		jr Z, +
		cp VDP_CHAR_BRICK_RIGHT+VDP_CHAR_ALT2_OFFSET
		jr Z, +

		ret

		+:
		jr break_brick_inner
		
	ret
	;================================================;



	;------------------------------------------------;
	;Macro for showing score information
	;Expects IX to contain location of BCD numerals
	;Expects VDP to be ready for writing
	;------------------------------------------------;
	.MACRO printbcd
		-: ;Start of character loop
			;Get the top bits		
			ld A, (IX+0)      ;Load byte of score into A
			srl A           ;Shift down to high 4 bits only
			srl A
			srl A
			srl A
			add A, VDP_CHAR_NUMERAL_BASE ;Offset from first numeral

			out (VDP_DATA_PORT), A ;Write character index
			ld A, 0
			out (VDP_DATA_PORT), A ;Write flags (0)

			;Get the bottom bits
			ld C, (IX+0)      ;Load byte of score into C
			ld A, $0F         ;Load low-bits mask into A
			and C             ;A now contains low bits of score
			add A, VDP_CHAR_NUMERAL_BASE ;Offset from the first numeral character

			out (VDP_DATA_PORT), A ;Write the byte of score
			ld A, 0
			out (VDP_DATA_PORT), A ;Write the flags (0)

			inc IX
		djnz - ;Loop if we have more characters to print
	.ENDM
	;================================================;
	
	
	;------------------------------------------------;
	;Function to print new score data on the screen
	;------------------------------------------------;
	update_scores:
			
		;Set placement of character
		vdp_ready_vram_write_address VDP_SCREEN_MAP_LOC+(2* ((13*32)+24) )
		
		;Set initial address of top score data
		ld IX, topscore_bcd ;RAM address
		;We will write 3 pairs of numerals, high-end first
		ld B, 3
	
		printbcd
		
		;Do the same for the current score
		vdp_ready_vram_write_address VDP_SCREEN_MAP_LOC+(2* ((10*32)+24) )   
		ld IX, score_bcd
		ld B, 3
		
		printbcd
		
		;Print the stage number
		vdp_ready_vram_write_address VDP_SCREEN_MAP_LOC+(2* ((7*32)+29) )   
		ld IX, next_stage_bcd
		ld B, 1
		
		printbcd

	ret
	;================================================;
	
	
	;------------------------------------------------;
	;Function to write new data into sprite table
	;------------------------------------------------;
	update_sprites:
		;Write vertical positions
		vdp_ready_vram_write_address VDP_SPRITE_TAB_LOC
		ld C, VDP_DATA_PORT
		
		;4 sprites of paddle - depending on tilt
		ld A, (paddle_tilt)
		ld B, A
		cp -1
		ld A, 159
		jr NZ, +
			inc A
		+:

		sub B
		out (C), A
		
		add A, B
		out (C), A
		add A, B
		out (C), A
		add A, B
		out (C), A
	
		
		
		;1 sprite of ball - position offscreen if in gutter
		ld A, (gutter_wait)
		cp 0
		ld A, 193
		jr NZ, +
			ld A, (ball_y) ;If gutter_wait is 0, use the true coordinate
		+:
		dec A
		out (C), A
		
		;3 sprites to indicate spare balls
		;position off-screen if not remaining
		ld A, (spare_balls)
		cp 3
		jr NZ, +
			;3 spare balls
			ld A, 172
			out (C), A
			nop
			out (C), A
			nop
			out (C), A
			
			jr _ballspr_done
		+:
		cp 2
		jr NZ, +
			;2 spare balls
			ld A, 172
			out (C), A
			nop
			out (C), A
			ld A, 193
			out (C), A
			
			jr _ballspr_done
		+:
		cp 1
		jr NZ, +
			;1 spare ball
			ld A, 172
			out (C), A
			ld A, 193
			out (C), A
			nop
			out (C), A
			
			jr _ballspr_done
		+:
		
		;0 spare balls
		ld A, 193
		out (C), A
		nop
		out (C), A
		nop
		out (C), A
		
		_ballspr_done:
		
		;Terminator
		ld A, $D0
		out (C), A
		
		
		;Write horizontal positions and character codes
		vdp_ready_vram_write_address VDP_SPRITE_TAB_LOC+$80
		ld C, VDP_DATA_PORT
		
		;4 sprites of paddle
		ld A, (paddle_x)
		ld B, VDP_CHAR_SPRITE_PADDLE
		
		out (C), A
		nop
		out (C), B
		
		add A, 8
		inc B
		out (C), A
		nop
		out (C), B
		
		add A, 8
		inc B
		out (C), A
		nop
		out (C), B
		
		add A, 8
		inc B
		out (C), A
		nop
		out (C), B
		
		;1 sprite of ball
		ld A, (ball_x)
		ld B, VDP_CHAR_SPRITE_BALL0
		out (C), A
		nop
		out (C), B
		
		;3 sprites of spare balls
		ld A, 192
		ld B, VDP_CHAR_SPRITE_BALL0
		out (C), A
		nop
		out (C), B
		add A, 10
		out (C), A
		nop
		out (C), B
		add A, 10
		out (C), A
		nop
		out (C), B
		
		ret
	;================================================;


	;------------------------------------------------;
	;Gamay Obera
	;------------------------------------------------;
	do_gameover:
	
		call preserve_topscore
	
		rst $00
	
	ret ;Never actually called
	;================================================;

.ENDS ;text


;Put this in bank/slot 0 too
.BANK 0 SLOT 0
.SECTION "data" ALIGN 2 FREE


	.ASCIITABLE
		MAP "q" = VDP_CHAR_BRICK_SINGLE
		MAP "w" = VDP_CHAR_BRICK_LEFT
		MAP "e" = VDP_CHAR_BRICK_FILL
		MAP "r" = VDP_CHAR_BRICK_RIGHT
		
		MAP "a" = VDP_CHAR_BRICK_SINGLE + VDP_CHAR_ALT1_OFFSET
		MAP "s" = VDP_CHAR_BRICK_LEFT   + VDP_CHAR_ALT1_OFFSET
		MAP "d" = VDP_CHAR_BRICK_FILL   + VDP_CHAR_ALT1_OFFSET
		MAP "f" = VDP_CHAR_BRICK_RIGHT  + VDP_CHAR_ALT1_OFFSET
		
		MAP "z" = VDP_CHAR_BRICK_SINGLE + VDP_CHAR_ALT2_OFFSET
		MAP "x" = VDP_CHAR_BRICK_LEFT   + VDP_CHAR_ALT2_OFFSET
		MAP "c" = VDP_CHAR_BRICK_FILL   + VDP_CHAR_ALT2_OFFSET
		MAP "v" = VDP_CHAR_BRICK_RIGHT  + VDP_CHAR_ALT2_OFFSET
		
		MAP " " = 0
	.ENDA

	;Each row of bricks is 22 characters wide
	pattern_table:
		.ASC "                      ----------" ;Blank

		.ASC "  wrwrwrwrwrwrwrwrwr  ----------" ;Solid 2-wide A
		.ASC "  sfsfsfsfsfsfsfsfsf  ----------" ;Solid 2-wide B
		.ASC "  xvxvxvxvxvxvxvxvxv  ----------" ;Solid 2-wide C
		.ASC " wrwrwrwrwrwrwrwrwrwr ----------" ;Solid 2-wide A (offset)
		.ASC " sfsfsfsfsfsfsfsfsfsf ----------" ;Solid 2-wide B (offset)
		.ASC " xvxvxvxvxvxvxvxvxvxv ----------" ;Solid 2-wide C (offset)

		.ASC "  wrxvwrsfwrsfwrxvwr  ----------"
		.ASC "sf wer wr xv wr wer sf----------"
		.ASC "werqsdf xcvxcv sdfqwer----------"
		.ASC " q a z weerweer z a q ----------"
		.ASC "q xv wer sddf wer xv q----------"

	.ASCIITABLE
	.ENDA		

	;Each stage consists of 16 rows - indexes into pattern table
	;Last row should always be 0... strange bug with the block counting...
	stage_table:
		.DB  0,11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ;Stage 1 is the start, not stage 0
		.DB  0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0
		.DB  0, 1, 4, 1, 4, 0, 0, 1, 4, 1, 4, 0, 0, 0, 0, 0
		.DB  0, 1, 2, 3, 0, 1, 2, 3, 0, 1, 2, 3, 0, 0, 0, 0
		.DB  0, 1, 4, 1, 4, 0, 2, 5, 2, 5, 0, 3, 6, 3, 6, 0
		.DB  0, 2, 1, 5, 0, 0, 4, 5, 0, 0, 1, 2, 0, 0, 3, 0
		.DB  0, 1, 2, 3, 2, 1, 0, 0, 1, 2, 3, 2, 1, 0, 4, 0
		.DB  0, 1, 5,11, 7, 8, 2, 8, 9, 7, 6,10,11, 4, 7, 0
		.DB  0, 4, 7, 0, 2, 4, 0, 1, 1, 2, 8, 9,10, 0, 0, 0
		.DB  0, 8, 5, 8,10, 0, 0,10, 5, 8, 5, 0, 0, 1, 0, 0
		.DB  0, 2, 4, 0, 2, 4, 0, 2, 5, 0, 0,11, 0, 0, 0, 0
		.DB  0, 1, 2, 3, 4, 4, 4, 3, 2, 1, 9, 0, 0, 9, 0, 0
		.DB  0, 4, 2, 4, 0, 6, 5, 3, 0, 3, 5, 6, 0, 0, 0, 0
		.DB  0,10,11, 6, 5, 6, 5, 0, 0, 6, 5, 3, 0, 0, 0, 0 
		.DB  0, 9, 9, 9, 9, 9, 9, 8, 9, 9, 9, 9, 9, 9, 9, 0
		.DB  0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0


	;Include sprite graphics
	sprites_pal:
		.INCBIN "src/sprites.pal.bin"
	sprites:
		.INCBIN "src/sprites.bin"
		

.ENDS ;data


;Background graphics sets go in slot 2. By default, use the title screen.
.BANK 2 SLOT 2
.SECTION "titlesection" ALIGN 2 FREE

	title_pal:
		.INCBIN "src/title.pal.bin"
	title:
		.INCBIN "src/title.bin"

.ENDS ;titlesection

;banks 3 and above are the ones not mapped by default

;Background graphics will get swapped into slot 1, overriding the title screen
.BANK 3 SLOT 2
.SECTION "bg0" ALIGN 2 FREE

	bg0_pal:
		.INCBIN "src/bg0.pal.bin"
	bg0:
		.INCBIN "src/bg0.bin"
		
	
.ENDS ;bg0

;We actually address bg0 and bg0_pal regardless of which background is in use
;The only difference is in the bank number we switch to
.BANK 4 SLOT 2
.SECTION "bg1" ALIGN 2 FREE

	bg1_pal:
		.INCBIN "src/bg1.pal.bin"
	bg1:
		.INCBIN "src/bg1.bin"
		
.ENDS ;bg1

.BANK 5 SLOT 2
.SECTION "bg2" ALIGN 2 FREE
	
	bg2_pal:
		.INCBIN "src/bg2.pal.bin"
	bg2:
		.INCBIN "src/bg2.bin"
		
.ENDS ;bg2

.BANK 6 SLOT 2
.SECTION "bg3" ALIGN 2 FREE

	bg3_pal:
		.INCBIN "src/bg2.pal.bin"
	bg3:
		.INCBIN "src/bg2.bin"
		
.ENDS ;bg3

.BANK 7 SLOT 2
.SECTION "bg4" ALIGN 2 FREE
	
.ENDS ;bg4