;SMS video definitions and macros

.DEFINE VDP_COMMAND_PORT $BF
.DEFINE VDP_DATA_PORT $BE

.DEFINE VDP_SCREEN_MAP_LOC $3800 ;Defined by VDP register R2
.DEFINE VDP_SPRITE_TAB_LOC $3F00 ;Defined by VDP register R5
.DEFINE VDP_SECOND_CHAR_PAGE_LOC $2000

;macro to set a VDP register
;first input: register to set
;second input: value
.MACRO vdp_set_register
	
	
	ld A, \2
	out (VDP_COMMAND_PORT), A ;write data first
	nop
	ld A, \1 | $80
	out (VDP_COMMAND_PORT), A ;then write register address
	
	
.ENDM

;macro to write a VDP color
;first input: color to write
;second input: value
.MACRO vdp_set_color
	
	ld A, \1
	out (VDP_COMMAND_PORT), A
	nop
	ld A, $C0
	out (VDP_COMMAND_PORT), A
	nop
	ld A, \1
	out (VDP_DATA_PORT), A
	
.ENDM

;macro to load a background palette
;expects HL loaded with address
.MACRO vdp_load_palette_bg
	
	ld A, $00
	out (VDP_COMMAND_PORT), A
	nop
	ld A, $C0
	out (VDP_COMMAND_PORT), A
	

	;we're loading 16 entries
	ld B, 16
	;we're outputting to the VDP data port
	ld C, VDP_DATA_PORT
	
	otir
		
.ENDM

;macro to load a foreground palette
;first input: memory address of palette
.MACRO vdp_load_palette_fg
	
	
	ld A, $10
	out (VDP_COMMAND_PORT), A
	nop
	ld A, $C0
	out (VDP_COMMAND_PORT), A

	ld B, 16
	ld C, VDP_DATA_PORT
	
	otir
	
	
.ENDM

;macro to write some bytes to VRAM
;first input: location in VRAM
;expects HL and B to contain array information (address, size)
.MACRO vdp_write_vram
	
	
	;write low byte of address
	ld A, <\1
	out (VDP_COMMAND_PORT), A
	nop
	;write high byte of address 
	ld A, >\1 | $40
	out (VDP_COMMAND_PORT), A
	nop

	ld C, VDP_DATA_PORT
	
	otir
	
	

.ENDM

;macro to write lots of data to vram (multiple writes of 256 bytes)
;first input: location in VRAM
;second input: how many 256-byte blocks to write
;expects HL to contain array information
.MACRO vdp_write_vram_256
	
	;write low byte of address
	ld A, <\1
	out (VDP_COMMAND_PORT), A
	nop
	;write high byte of address 
	ld A, >\1 | $40
	out (VDP_COMMAND_PORT), A
	nop

	ld C, VDP_DATA_PORT
	
	ld A, \2
-:
	ld B, 0
	otir
	dec A
	jr nz, -


.ENDM
	
;just sets up an address
.MACRO vdp_ready_vram_write_address
	;write low byte of address
	ld A, <\1
	out (VDP_COMMAND_PORT), A
	nop
	;write high byte of address 
	ld A, >\1 | $40
	out (VDP_COMMAND_PORT), A
	nop
.ENDM


;Loads a single character into the VDP
;HL should be loaded with address in CPU space
;Input to the macro is the VRAM address
.MACRO vdp_load_character
	vdp_ready_vram_write_address \1
	
	;Writing 32 bytes out the VDP data port, from HL (pre-loaded)
	ld C, VDP_DATA_PORT
	ld B, 32
	
	otir
.ENDM

;Wait for vertical blanking
.MACRO vdp_wait_vbl
	ld HL, frame_count
	ld A, (HL)
	-: ;Loop here until the interrupt handler changes the frame count
		ei
		halt
		cp (HL)
	jr Z, - ;Loop if the new and old frame counts are the same
	di
.ENDM


;takes a 2bpp sprite from ROM and unpacks it into main RAM
;IX should be set to the location in memory to read from
;IY is location to write to
;parameter 1 is the color offset to apply
;parameter 2 is how many sprites to unpack
.MACRO sprite_unpack_2bpp_to_4bpp

	ld A, \2 * 16 ;16 lines	
--:
	push AF

	;grab the packed sprite line in BC
	ld C, (IX+0)
	ld B, (IX+1)
	
	;prepare output in DEHL
	ld D, 0
	ld E, 0
	ld H, 0
	ld L, 0

	ld A, 8
-:
	push AF
	
	;pull the lowest color and put it in the accumulator
	ld A, $03
	and C
	
	;apply color offset
	add A, \1
	
	;accumulator is now the color of the pixel. if appropriate bit is set, toggle output
	bit 0, A
	jr z, +
	set 0, D
+:
	bit 1, A
	jr z, +
	set 0, E
+:
	bit 2, A
	jr z, +
	set 0, H
+:
	bit 3, A
	jr z, +
	set 0, L
+:
	;rotate the 2bpp line right
	RR B ;stores carry
	RR C ;uses carry
	RR B 
	RR C 
	
	;rotate our output right
	RRC D ;no carries
	RRC E
	RRC H
	RRC L

	pop AF ;check loop iterator
	dec A
	jr nz, -

	;DEHL now contains bitplanes
	ld C, VDP_DATA_PORT
	
	ld (IY+0), D
	ld (IY+1), E
	ld (IY+2), H
	ld (IY+3), L
	
	inc IY
	inc IY
	inc IY
	inc IY

	inc IX
	inc IX
	
	pop AF ;check loop iterator
	dec A
	jr nz, --
.ENDM
